function I=coord_ac(N,maxd,gamma)
%
% I=coord_ac(N,d,gamma)
%
%   computes axis cross index sets containing all h\in\mathbb{Z}^d
%   with \|h\|_0=1; and |gamma_s.^(-1).*h_s|<=N for all s
%   
%
%   N:      refinement
%   d:      dimension 
%   gamma:  vector of weights of length d
%           sometimes it's better to use weights+eps to avoid round down
%           errors
%
% Copyright 2014 (c) Lutz Kaemmerer
%   
% See also coord_symhc, coord_lpball

    if prod(size(gamma))==1
        gamma=ones(1,maxd)*gamma;
    elseif(min(size(gamma))~=1 || max(size(gamma))<maxd)
        error('weights gamma not of right size');
    end

    I=double2int(zeros(1,maxd),gamma(1)*N);
    
    for d=1:maxd
        tmp=[-floor(gamma(d)*N):-1,1:floor(gamma(d)*N)];
        I=[I;zeros(length(tmp),d-1),tmp',zeros(length(tmp),maxd-d)];
    end
end