function I=coord_lpball(N,maxd,p,gamma,opt)%
% I=coord_lpball(N,d,p,gamma,opt)
%
%   computes lp-ball index sets containing all h\in\mathbb{Z}^d
%   with
%   \prod_{s=1}^d\max(1,|h_s|/gamma_s)\le N
%
%   N:      refinement
%   d:      dimension 
%   p:      parameter
%   gamma:  descending ordered vector of nonnegative weights of length d
%           1\ge gamma_1\ge \ldots \gamma_d>0
%           sometimes it's better to use weights+eps to avoid round down
%           errors
%   opt.parity
%           even: only even indices
%           odd:  only odd indices (0 is even and odd)
%           oddodd: only oddodd indices (0 is not oddodd)
%           none: default
%
% Copyright 2014 (c) Lutz Kaemmerer
%   
% See also coord_symhc, coord_ac

    if(nargin<5)
        opt=0;
    end

    if prod(size(gamma))==1
        gamma=ones(1,maxd)*gamma;
    elseif(min(size(gamma))~=1 || max(size(gamma))<maxd)
        error('weights gamma not of right size');
    end

    a1=(0:N*gamma(1))';
    a1=double2int(a1,gamma(1)*N);
    if(isfield(opt,'parity'))
        if(strcmp(opt.parity,'even')==1)
            ind=find(mod(a1,2)==0);
        elseif(strcmp(opt.parity,'odd')==1)
            ind=[find(a1==0);find(mod(a1,2)==1)];
        elseif(strcmp(opt.parity,'oddodd')==1)
            ind=find(mod(a1,2)==1);
        end
        a1=a1(ind);
    end
    weight=(double(a1)./gamma(1)).^p;
 
for d=2:maxd
    %fprintf('d=%d,\n',d);
    a2=[];
    a2=double2int(a2,gamma(1)*N);
    weight2=[];

    for l=1:size(a1,1)
        if(weight(l)~=0)
            len=(N^p-weight(l)+eps)^(1/p)*gamma(d);
        else
            len=N*gamma(d);
        end
        tmp=double2int(0:len,gamma(d)*N);
        if(isfield(opt,'parity') && length(tmp)>0)
            if(strcmp(opt.parity,'even')==1)
                ind=find(mod(tmp,2)==0);
            elseif(strcmp(opt.parity,'odd')==1)
                ind=find(mod(tmp,2)==1);
                ind0=find(tmp==0);
                ind=[ind0;ind];
            elseif(strcmp(opt.parity,'oddodd')==1)
                ind=find(mod(tmp,2)==1);
            end
            tmp=tmp(ind);
        end
        if(length(tmp)>0)
            a2=[a2;double2int([(ones(1,length(tmp))'*double(a1(l,:))),tmp'], gamma(d)*N)];
            tmp=(double(tmp)/gamma(d)).^p;
            weight2=[weight2;weight(l)+tmp'];
        end
    end
    a1=a2;
    weight=weight2;
end
clear a2 weight weight2 len tmp;

I=sym_set(a1);
end