function [I]=coord_symhc(N,maxd,gamma,opt)
%
% I=coord_symhc(N,d,gamma,opt)
%
%   computes hyperbolic cross index sets containing all h\in\mathbb{Z}^d
%   with
%   \prod_{s=1}^d\max(1,|h_s|/gamma_s)\le N
%
%   N:      refinement
%   d:      dimension 
%   gamma:  descending ordered vector of nonnegative weights of length d
%           1\ge gamma_1\ge \ldots \gamma_d>0
%           sometimes it's better to use weights+eps to avoid round down
%           errors
%   opt.parity
%           even: only even indices
%           odd:  only odd indices (0 is even and odd)
%           oddodd: only oddodd indices (0 is not oddodd)
%           none: default
%
% Copyright 2014 (c) Lutz Kaemmerer
%   
% See also coord_ac, coord_lpball


    if(nargin<4)
        opt=0;
    end

    if prod(size(gamma))==1
        gamma=ones(1,maxd)*gamma;
    elseif(min(size(gamma))~=1 || max(size(gamma))<maxd)
        error('weights gamma not of right size');
    end

    a1=double2int((0:N*gamma(1))',gamma(1)*N);
    if(isfield(opt,'parity'))
        if(strcmp(opt.parity,'even')==1)
            ind=find(mod(a1,2)==0);
        elseif(strcmp(opt.parity,'odd')==1)
            ind=[find(a1==0);find(mod(a1,2)==1)];
        elseif(strcmp(opt.parity,'oddodd')==1)
            ind=find(mod(a1,2)==1);
        end
        a1=a1(ind);
    end
    weight=max(1,double(a1)./gamma(1));
    
    
for d=2:maxd
    a2=[];
    a2=double2int(a2,gamma(1)*N);
    weight2=[];

    for l=1:size(a1,1)
        tmp=double2int(0:N*gamma(d)/weight(l),gamma(1)*N);
        if(isfield(opt,'parity') && length(tmp)>0)
            if(strcmp(opt.parity,'even')==1)
                ind=find(mod(tmp,2)==0);
            elseif(strcmp(opt.parity,'odd')==1)
                ind=find(mod(tmp,2)==1);
                ind0=find(tmp==0);
                ind=[ind0;ind];
            elseif(strcmp(opt.parity,'oddodd')==1)
                ind=find(mod(tmp,2)==1);
            end
            tmp=tmp(ind);
        end
        a2=[a2;double2int([(ones(1,length(tmp))'*double(a1(l,:))),tmp'], gamma(1)*N)];
        tmp=max(1,double(tmp)/gamma(d));
        weight2=[weight2;weight(l)*tmp'];
    end
    a1=a2;
    weight=weight2;
end

clear a2 weight weight2 ind ind0 tmp;

if(prod(size(a1))>0)
    I=sym_set(a1);
else
    I=zeros(0,maxd);
end

end