function [rneu,M,exitflag,output]=gs_search(I,C,rstart,opt)
%
%   [r,M,exitflag,output]=gs_search(I,C,rstart,opt)
%
%   Input:
%
%   I:      frequency index set
%   C:      target condition number C=10 if omitted
%   rstart: starting vector for fminsearch, rand(size(I,2),1) if omitted
%   opt:    options of fminsearch
%
%   Output:
%
%   generating vector r and lattice size M such that
%   \Lambda(r,M)={jr, j=0:M-1} is reconstructing generated set of I
%   and the Fourier matrix A=exp(2*pi*1i*(0:M1-1)'*(double(I)*r')')
%   has condition number of at most sqrt(C).
%
%   exitfag and output -> see fminsearch
%
% Copyright 2014 (c) Lutz Kaemmerer
%   
% See also fminsearch, gsfft, gsifft, lfft, r1l_cbc_search

    I=double(I);
   
    if(nargin<4)
        opt.Display='verbose';
        %opt.Display='final';
        opt.MaxIter=10000;
        %opt.MaxFunEvals=10000*size(I,2);
        opt.MaxFunEvals=3000;
        opt.TolX=10^(-10);
        opt.TolFun=10^(-8)*size(I,1); % TolFun relative to minimum
    end
    if(nargin<2)
        C=7;   % standard target condition number
    elseif(C<=1)
        error('C>1 required');
    end
    
    if(nargin>=3)
        if(length(rstart)<size(I,2))
            r(1:length(rstart))=reshape(rstart,1,length(rstart));
            r(length(rstart)+1:size(I,2))=rand(1,size(I,2)-length(rstart));
        else
            r(1:length(rstart))=rstart;
        end    
    end
    r=rand(1,size(I,2));
    [rneu,f,exitflag,output]=fminsearch(@(x) rhovalue(I,x),r,opt);
    M=ceil((C+1)/(C-1)*f);
    rneu=mod(rneu,1);
end

function ret=rhovalue(I,r)

    y=sort(mod(I*r',1));
    
    y=sort([y(2:length(y));y(1)+1]-y);
    
    s=sum(y(1:floor(length(y)/2)));
    ret=0;
    for j=floor(length(y)/2):-1:1
        ret=ret+1/s;
        s=s-y(j);
    end
end
