function [f]=gsdft(I,r,M,fhat,tflag)
%
%   [f]=gsdft(I,r,M,fhat,tflag)
%
%   naive conputation of the ldft using loops
%   
%   I:      frequency index set
%   r:      generating integer vector
%   M:      lattice size
%   fhat:   Fourier coefficients fhat(j)=fhat_I(j,:) with tflag='notransp'
%           Function values fhat(j)=f(mod(j*r,1)) with tflag='transp'
%   tflag:  'notransp' -> Evaluation of 
%               f(j+1)=sum_{k\in I}fhat(k)*e^(-2*pi*i*j*k'r), j=0:M-1
%           'transp' -> Evaluation of transposed
%               f(l)=sum_{j=0}^M fhat(j)*e^(2*pi*i*j*k(l)'r),
%                                           k(l)=I(l,:), l=1:size(h,1)
%
% Copyright 2014 (c) Lutz Kaemmerer
%   
% See also gsfft, gsifft, ldft, lfft


    if(nargin<5)
        tflag='notransp';
    end
    I=double(I);
    
    switch(tflag)
        case 'transp'
            if(length(fhat)==M)
                f=zeros(size(I,1),1);
                for j=1:size(I,1)
                    for l=1:M
                        f(j)=f(j)+fhat(l)*exp(-2*pi*1i*(l-1)*I(j,:)*r');
                    end
                end
            else
                error('length of fhat has to be M for tflag="transp"');
            end
        otherwise
            if(length(fhat)==size(I,1))
                f=zeros(M,1);
                for j=1:size(I,1)
                    for l=1:M
                        f(l)=f(l)+fhat(j)*exp(2*pi*1i*(l-1)*I(j,:)*r');
                    end
                end
            else
                error('length of fhat has to be size(I,1) for tflag="notransp"');
            end
    end
end