function [f]=gsfft(I,r,M,f_hat,tflag)
%
% f=gsfft(I,r,M,fhat,tflag)
%
%   I:      frequency index set
%   r:      generating real vector
%   M:      lattice size
%   fhat:   Fourier coefficients f_hat(j)=f_hat_h(j,:) with tflag='notransp'
%           Function values f_hat(j)=f(mod(j*r/M,1)) with tflag='transp'
%   tflag:  'notransp' -> Evaluation of 
%
%               f(j+1)=sum_{k\in I}f_hat_k*e^(2*pi*i*j*k'*r), j=0:M-1
%
%           'transp' -> Evaluation of transposed
%
%               f(l)=sum_{j=0}^{M-1} f_hat(j)*e^(-2*pi*i*j*k(l)'r),
%                                           k(l)=I(l,:), l=1:size(I,1)
%
% Copyright 2014 (c) Lutz Kaemmerer
%
% See also: gsifft, lfft

    if(~isfloat(I))
        I=double(I);
    end
    switch(tflag)
        case 'notransp'
            f=gsfft_notransp(I,r,M,f_hat);
        case 'transp'
            f=gsfft_transp(I,r,M,f_hat);
        case 'both'
            f=gsfft_transp(I,r,M,gsfft_notransp(I,r,M,f_hat));
    end

end

function y=gsfft_notransp(I,r,M,fhat)

if min(size(r))~=1
    error('size r');
end

if size(I,2)~=max(size(r))
    error('dimension of r and elements of I not consistent');
end

if size(I,1)~=length(fhat)
    error('fhat has not the same number of elements as I');
end


if size(I,2)~=size(r,2)
    r=r';
end

y=mod(I*r',1);
y(y>=0.5)=y(y>=0.5)-1;

opt.method='gaussian';
%opt.method='fast_gaussian';
%opt.method='taylor';
opt.sigma=2;
opt.m=12;


y=nfft(fhat,y,2*M,opt,'transp');
y=y(M+1:2*M);
end

function y=gsfft_transp(I,r,M,f)
if min(size(r))~=1
	error('size r');
end
if size(I,2)~=max(size(r))
    error('dimension of r and elements in I');
end
if M~=length(f)
    error('dimension of fhat has to be M for transpose');
end
if size(I,2)~=size(r,2)
    r=r';
end

y=mod(I*r',1);
y(y>=0.5)=y(y>=0.5)-1;

opt.method='gaussian';
%opt.method='fast_gaussian';
%opt.method='taylor';
opt.sigma=2;
opt.m=12;

% ungerades M ausgleichen:
f=[zeros(M,1);f];
y=nfft(f,y,2*M,opt,'notransp');
end