function [fhat, numiter]=gsifft(I,r,M,f)
%
% [fhat,numiter]=gsifft(I,r,M,f)
%
%   I:      frequency index set
%   r:      generating real vector
%   M:      lattice size
%   f:      function values of f
%
%   computes the pseudoinverse of a gsfft, i.e.
%   fhat=(A'*A)*A'*f, A=exp(2*pi*1i*(0:M1-1)'*(double(I)*r')'),
%   using a simple conjugate gradient method
%   numiter is the number of iterations that the CG method needed
%
% Copyright 2014 (c) Lutz Kaemmerer
%
% See also: gsfft, lfft


    err=10^-10;
    f1=gsfft(I,r,M,f,'transp');
    [fhat,numiter]=dirtycg(@(x) gsfft(I,r,M,x,'both'),f1,err);
   
end

function [x,j] = dirtycg(fhandle,b,err)

    x=zeros(length(b),1);
    %r=b-fhandle(x);
    r=b;
    p=r;
    errakt=ceil(log10(sqrt(r'*r)));
    %fprintf('initial error 10 to the power of %d\n',errakt);
    rsold=r'*r;
 
    for j=1:length(b)
        Ap=fhandle(p);
        alpha=rsold/(p'*Ap);
        x=x+alpha*p;
        r=r-alpha*Ap;
        rsnew=r'*r;
        if sqrt(rsnew)<err
              break;
        end
        if(ceil(log10(sqrt(rsnew)))<=errakt-1)
            errakt=ceil(log10(sqrt(rsnew)));
            %fprintf('iteration %d actual error 10 to the power of %d\n',j,errakt);
        end
        p=r+rsnew/rsold*p;
        rsold=rsnew;
    end
end