function [f]=lfft(I,z,M,fhat,tflag)
%
% f=lfft(I,z,M,fhat,tflag)
%
%   I:      frequency index set
%   z:      generating integer vector
%   M:      lattice size
%   fhat:   Fourier coefficients fhat(j)=fhat_I(j,:) with tflag='notransp'
%           Function values fhat(j)=f(mod(j*z/M,1)) with tflag='transp'
%   tflag:  'notransp' -> Evaluation of 
%               f(j+1)=sum_{k\in I}fhat(k)*e^(-2*pi*i*j*k'z/M), j=0:M-1
%           'transp' -> Evaluation of transposed
%               f(l)=sum_{j=0}^M fhat(j)*e^(2*pi*i*j*k(l)'z/M),
%                                           k(l)=I(l,:), l=1:size(h,1)
%
%   Note: 
%
%       lfft(I,z,M,lfft(I,z,M,fhat,'notransp'),'transp')=M*fhat           
%       in the case that z and M generates a reconstruction rank-1 lattice
%       for I
%
% Copyright 2014 (c) Lutz Kaemmerer
%   
% See also gsfft, gsifft

    if(~isfloat(I))
        I=double(I);
    end
    switch(tflag)
        case 'notransp'
            f=lfft_notransp(I,z,M,fhat);
        case 'transp'
            f=lfft_transp(I,z,M,fhat);
    end
end

function f=lfft_notransp(I,z,M,f_hat)
    k=mod(I*z',M)+1;
    fhat1=accumarray(k,f_hat,[M,1],@sum);
    f=M*ifft(fhat1);
end

function fhat=lfft_transp(I,z,M,f)
    k=mod(I*z',M)+1;
    ghat=fft(f);
    fhat = ghat(k);
end