function [M]=lfft_lattice_size_reduce(I,Mmax,z,opt)
%
%
%   opt.Display:   
%       'off':      no display
%       'verbose':  display actual M
%   opt.Prime:
%       1: only primes
%       0: all M<Mmax
%   opt.searchmethod:
%       Increase:   Increase search
%       Bisection:     use only bisections
%       Decrease:   Decrease search
%   opt.minM:
%       positive integer - minimal allowed M, maybe known from
%       theory
%
%   Lutz Kmmerer 2014

Iz=double(I)*z';
Mmax=min(Mmax,max(Iz)-min(Iz)+1);

if(nargin==3)
    opt.Display='off';
    opt.Prime=0;
    opt.searchmethod='Increase';
    opt.Bisection_Successors=0;
    opt.minM=length(Iz);
else
    if(~(isfield(opt,'Prime') && opt.Prime==1))
        opt.Prime=0;
    end
    if(~(isfield(opt,'Display') && strmatch(opt.Display, char('off', 'verbose'),'exact')>0))
        opt.Display='off';
    end
    if(~(isfield(opt,'searchmethod') && strmatch(opt.searchmethod, char('Increase', 'Bisection', 'Decrease'),'exact')>0))
        opt.searchmethod='Increase';
    end
    if(~(isfield(opt,'minM') && opt.minM>=length(Iz)))
        opt.minM=length(Iz);
    end
    if(~isfield(opt,'Bisection_Successors'))
        opt.Bisection_Successors=min([max(sqrt(Mmax-opt.minM),50),Mmax-opt.minM,1000]);
    end
end    

switch(opt.Prime)
    case 0
            candidates=[opt.minM,Mmax];
            p = inline('t');
    case 1
        if(Mmax<396738)
            p=primes(ceil(4/3*(Mmax+2)));
        else
            p=primes((1+1/25/log(Mmax)^2)*Mmax);
        end
        p=p(p<=Mmax);
        p=p(p>=opt.minM);
        candidates=[1,length(p)];
end


if(strcmp(opt.searchmethod,'Bisection')==1)
   % make 20 Bisections with bs=min(max(sqrt(Mmax-|I|),50),Mmax-|I|,1000)
   % many successive candidates
   jotu=max(candidates);
   jotl=min(candidates);
   jot=ceil((jotu+jotl)/2);
   bs=opt.Bisection_Successors;
   while(jotu>jotl)
       clear lum;
       lum=[];
       for(r=max(jotl,jot-floor(bs/2)):min(jotu,jot+floor(bs/2)))
           lum(r-jot+floor(bs/2)+1)=length(unique(mod(Iz,p(r))));
           if(lum(r-jot+floor(bs/2)+1)==length(Iz))
               jot=r;
               if(strcmp(opt.Display,'verbose')==1)
                   fprintf('actual lattice size: %d\n',p(jot));
               end
               break;
           end
       end
       if(r==jotu)
            break;
       end
       max(lum)
       [jotl,jotu]
       if(max(lum)==size(I,1))
            jotu=jot;
       else
            jotl=jot;
       end
       [jotl,jotu]
       jot=ceil((jotu+jotl)/2);
    end
    M=p(jotu);
else
    switch(opt.searchmethod)
        case 'Increase'
            for j=min(candidates):1:max(candidates)
                if (length(unique(mod(Iz,p(j))))==length(Iz))
                    M=p(j);
                    break;
                end
            end
        case 'Decrease'
            for j=max(candidates):-1:min(candidates)
                if (length(unique(mod(Iz,p(j))))==length(Iz))
                    M=p(j);
                    if(strcmp(opt.Display,'verbose')==1)
                        fprintf('actual lattice size: %d\n',M);
                    end
                end
            end
    end
end
end
